package cz.muni.fi.pv168.project.model;

import cz.muni.fi.pv168.project.data.category.Category;

import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.table.AbstractTableModel;
import java.time.LocalDate;

public class CategoryStatisticsTableModel extends AbstractTableModel implements ListDataListener {
    private final ListModel<Category> model;
    private LocalDate fromDate;
    private LocalDate toDate;

    public CategoryStatisticsTableModel(ListModel<Category> model) {
        fromDate = LocalDate.now();
        toDate = LocalDate.now();
        this.model = model;

        model.addListDataListener(this);
    }

    @Override
    public int getRowCount() {
        return model.getSize();
    }

    @Override
    public int getColumnCount() {
        return 2;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        var category = model.getElementAt(rowIndex);
        switch (columnIndex) {
            case 0:
                return category;
            case 1:
                return category.getTimeDurationFormatted(fromDate, toDate);
            default:
                throw new IndexOutOfBoundsException("Invalid column index: " + columnIndex);
        }
    }

    @Override
    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return "Category name";
            case 1:
                return "Time spent";
            default:
                throw new IndexOutOfBoundsException("Invalid column index: " + columnIndex);
        }
    }

    @Override
    public void intervalAdded(ListDataEvent listDataEvent) {
        fireTableRowsInserted(listDataEvent.getIndex0(), listDataEvent.getIndex1());
    }

    @Override
    public void intervalRemoved(ListDataEvent listDataEvent) {
        fireTableRowsDeleted(listDataEvent.getIndex0(), listDataEvent.getIndex1());
    }

    @Override
    public void contentsChanged(ListDataEvent listDataEvent) {
        fireTableDataChanged();
    }

    public void setFilter(LocalDate fromFilterDate, LocalDate toFilterDate) {
        this.fromDate = fromFilterDate;
        this.toDate = toFilterDate;
    }
}
